@@ -34,23 +34,13 @@ static List *sort(List *);
 
 extern bool lmatch(List *s, List *p) {
 	List *q;
-	int i;
-	bool okay;
 	if (s == NULL) {
 		if (p == NULL) /* null matches null */
 			return TRUE;
-		for (; p != NULL; p = p->n) { /* one or more stars match null */
-			if (*p->w != '\0') { /* the null string is a special case; it does *not* match () */
-				okay = TRUE;
-				for (i = 0; p->w[i] != '\0'; i++)
-					if (p->w[i] != '*' || p->m[i] != 1) {
-						okay = FALSE;
-						break;
-					}
-				if (okay)
-					return TRUE;
-			}
-		}
+		for (; p != NULL; p = p->n) /* one or more stars match null */
+			if (strspn(p->w, "*") == strlen(p->w) &&
+			    p->m != NULL && strlen(p->m) == strlen(p->w))
+				return TRUE;
 		return FALSE;
 	}
 	for (; s != NULL; s = s->n)
@@ -127,6 +117,7 @@ static List *dmatch(char *d, char *p, char *m) {
 	}
 
 	top = r = NULL;
+	if (*d == '\0') d = "/";
 	if ((dirp = opendir(d)) == NULL)
 		return NULL;
 	/* opendir succeeds on regular files on some systems, so the stat() call is necessary (sigh) */
@@ -216,12 +207,10 @@ static List *doglob(char *w, char *m) {
 	p = pattern;
 	md = metadir;
 	mp = metapattern;
-	if (*s == '/')
-		while (*s == '/')
-			*d++ = *s++, *md++ = *m++;
-	else
-		while (*s != '/' && *s != '\0')
-			*d++ = *s++, *md++ = *m++; /* get first directory component */
+	while (*s != '/' && *s != '\0') {
+		*d++ = *s++; /* get first directory component */
+		*md++ = *m++;
+	}
 	*d = '\0';
 	/*
 	   Special case: no slashes in the pattern, i.e., open the current directory.
