@@ -8,30 +8,31 @@
    and to strip out unneeded functionality.
 */
 
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/param.h>
-#include <errno.h>
 #include "rc.h"
 
+#include <errno.h>
+#include <sys/stat.h>
+
+#include "getgroups.h"
+
 #define X_USR 0100
 #define X_GRP 0010
 #define X_OTH 0001
 #define X_ALL (X_USR|X_GRP|X_OTH)
 
-extern int stat(const char *, struct stat *);
-
 static bool initialized = FALSE;
-static int uid, gid;
+static uid_t uid;
+static gid_t gid;
 
-#ifdef NGROUPS
-static int ngroups, gidset[NGROUPS];
+#if HAVE_GETGROUPS
+static int ngroups;
+static GETGROUPS_T *gidset;
 
 /* determine whether gid lies in gidset */
 
-static int ingidset(int g) {
+static int ingidset(gid_t g) {
 	int i;
-	for (i = 0; i < ngroups; i++)
+	for (i = 0; i < ngroups; ++i)
 		if (g == gidset[i])
 			return 1;
 	return 0;
@@ -55,7 +56,7 @@ static bool rc_access(char *path, bool verbose) {
 		mask = X_ALL;
 	else if (uid == st.st_uid)
 		mask = X_USR;
-#ifdef NGROUPS
+#if HAVE_GETGROUPS
 	else if (gid == st.st_gid || ingidset(st.st_gid))
 #else
 	else if (gid == st.st_gid)
@@ -75,7 +76,7 @@ static bool rc_access(char *path, bool verbose) {
 
 extern char *which(char *name, bool verbose) {
 	static char *test = NULL;
-	static SIZE_T testlen = 0;
+	static size_t testlen = 0;
 	List *path;
 	int len;
 	if (name == NULL)	/* no filename? can happen with "> foo" as a command */
@@ -84,15 +85,25 @@ extern char *which(char *name, bool verbose) {
 		initialized = TRUE;
 		uid = geteuid();
 		gid = getegid();
-#ifdef NGROUPS
-		ngroups = getgroups(NGROUPS, gidset);
+#if HAVE_GETGROUPS
+#if HAVE_POSIX_GETGROUPS
+		ngroups = getgroups(0, (GETGROUPS_T *)0);
+		if (ngroups < 0) {
+			uerror("getgroups");
+			rc_exit(1);
+		}
+#else
+			ngroups = NGROUPS;
+#endif
+		gidset = ealloc(ngroups * sizeof(GETGROUPS_T));
+		getgroups(ngroups, gidset);
 #endif
 	}
 	if (isabsolute(name)) /* absolute pathname? */
 		return rc_access(name, verbose) ? name : NULL;
 	len = strlen(name);
 	for (path = varlookup("path"); path != NULL; path = path->n) {
-		SIZE_T need = strlen(path->w) + len + 2; /* one for null terminator, one for the '/' */
+		size_t need = strlen(path->w) + len + 2; /* one for null terminator, one for the '/' */
 		if (testlen < need) {
 			efree(test);
 			test = ealloc(testlen = need);
